@@ -1,10 +1,11 @@
 # -*- coding: utf-8 -*-
-# Copyright © 2015-2018 ACSONE SA/NV
+# Copyright © 2015-2021 ACSONE SA/NV
 # License LGPLv3 (http://www.gnu.org/licenses/lgpl-3.0-standalone.html)
 
 import email.parser
 import io
 import os
+import re
 from distutils.core import DistutilsSetupError
 from email.message import Message
 from warnings import warn
@@ -12,14 +13,22 @@ from warnings import warn
 import setuptools
 
 from . import base_addons, external_dependencies
-from .git_postversion import STRATEGY_99_DEVN, STRATEGY_P1_DEVN, get_git_postversion
+from .git_postversion import (
+    STRATEGY_99_DEVN,
+    STRATEGY_DOT_N,
+    STRATEGY_P1_DEVN,
+    get_git_postversion,
+)
 from .manifest import is_installable_addon, read_manifest
 
+METADATA_NAME_RE = re.compile(r"^odoo(\d*)-addon-(?P<addon_name>.*)$")
+
+
 ODOO_VERSION_INFO = {
     "7.0": {
         "odoo_dep": "openerp>=7.0a,<8.0a",
         "base_addons": base_addons.openerp7,
-        "pkg_name_pfx": "openerp7-addon-",
+        "pkg_name_pfx": "openerp7-addon",
         "addons_ns": "openerp_addons",
         "namespace_packages": ["openerp_addons"],
         "python_requires": "~=2.7",
@@ -29,7 +38,7 @@ ODOO_VERSION_INFO = {
     "8.0": {
         "odoo_dep": "odoo>=8.0a,<9.0a",
         "base_addons": base_addons.odoo8,
-        "pkg_name_pfx": "odoo8-addon-",
+        "pkg_name_pfx": "odoo8-addon",
         "addons_ns": "odoo_addons",
         "namespace_packages": ["odoo_addons"],
         "python_requires": "~=2.7",
@@ -39,7 +48,7 @@ ODOO_VERSION_INFO = {
     "9.0": {
         "odoo_dep": "odoo>=9.0a,<9.1a",
         "base_addons": base_addons.odoo9,
-        "pkg_name_pfx": "odoo9-addon-",
+        "pkg_name_pfx": "odoo9-addon",
         "addons_ns": "odoo_addons",
         "namespace_packages": ["odoo_addons"],
         "python_requires": "~=2.7",
@@ -49,7 +58,7 @@ ODOO_VERSION_INFO = {
     "10.0": {
         "odoo_dep": "odoo>=10.0,<10.1dev",
         "base_addons": base_addons.odoo10,
-        "pkg_name_pfx": "odoo10-addon-",
+        "pkg_name_pfx": "odoo10-addon",
         "addons_ns": "odoo.addons",
         "namespace_packages": ["odoo", "odoo.addons"],
         "python_requires": "~=2.7",
@@ -59,7 +68,7 @@ ODOO_VERSION_INFO = {
     "11.0": {
         "odoo_dep": "odoo>=11.0a,<11.1dev",
         "base_addons": base_addons.odoo11,
-        "pkg_name_pfx": "odoo11-addon-",
+        "pkg_name_pfx": "odoo11-addon",
         "addons_ns": "odoo.addons",
         "namespace_packages": None,
         "python_requires": ", ".join(
@@ -71,7 +80,7 @@ ODOO_VERSION_INFO = {
     "12.0": {
         "odoo_dep": "odoo>=12.0a,<12.1dev",
         "base_addons": base_addons.odoo12,
-        "pkg_name_pfx": "odoo12-addon-",
+        "pkg_name_pfx": "odoo12-addon",
         "addons_ns": "odoo.addons",
         "namespace_packages": None,
         "python_requires": ">=3.5",
@@ -81,7 +90,7 @@ ODOO_VERSION_INFO = {
     "13.0": {
         "odoo_dep": "odoo>=13.0a,<13.1dev",
         "base_addons": base_addons.odoo13,
-        "pkg_name_pfx": "odoo13-addon-",
+        "pkg_name_pfx": "odoo13-addon",
         "addons_ns": "odoo.addons",
         "namespace_packages": None,
         "python_requires": ">=3.5",
@@ -91,18 +100,40 @@ ODOO_VERSION_INFO = {
     "14.0": {
         "odoo_dep": "odoo>=14.0a,<14.1dev",
         "base_addons": base_addons.odoo14,
-        "pkg_name_pfx": "odoo14-addon-",
+        "pkg_name_pfx": "odoo14-addon",
         "addons_ns": "odoo.addons",
         "namespace_packages": None,
         "python_requires": ">=3.6",
         "universal_wheel": False,
         "git_postversion_strategy": STRATEGY_P1_DEVN,
     },
+    "15.0": {
+        "odoo_dep": "odoo>=15.0a,<15.1dev",
+        "base_addons": base_addons.odoo15,
+        "pkg_name_pfx": "odoo-addon",
+        "pkg_version_specifier": ">=15.0dev,<15.1dev",
+        "addons_ns": "odoo.addons",
+        "namespace_packages": None,
+        "python_requires": ">=3.8",
+        "universal_wheel": False,
+        "git_postversion_strategy": STRATEGY_DOT_N,
+    },
+    "16.0": {
+        "odoo_dep": "odoo>=16.0a,<16.1dev",
+        "base_addons": base_addons.odoo16,
+        "pkg_name_pfx": "odoo-addon",
+        "pkg_version_specifier": ">=16.0dev,<16.1dev",
+        "addons_ns": "odoo.addons",
+        "namespace_packages": None,
+        "python_requires": ">=3.8",
+        "universal_wheel": False,
+        "git_postversion_strategy": STRATEGY_DOT_N,
+    },
 }
 
 
 def _get_odoo_version_info(addons_dir, odoo_version_override=None):
-    """ Detect Odoo version from an addons directory """
+    """Detect Odoo version from an addons directory"""
     odoo_version_info = None
     addons = os.listdir(addons_dir)
     for addon in addons:
@@ -132,7 +163,7 @@ def _get_version(
     git_post_version=True,
     post_version_strategy_override=None,
 ):
-    """ Get addon version information from an addon directory """
+    """Get addon version information from an addon directory"""
     version = manifest.get("version")
     if not version:
         warn("No version in manifest in %s" % addon_dir)
@@ -192,17 +223,23 @@ def _get_author_email(manifest):
 
 
 def make_pkg_name(odoo_version_info, addon_name):
-    name = odoo_version_info["pkg_name_pfx"] + addon_name
+    name = odoo_version_info["pkg_name_pfx"] + "-" + addon_name
     return name
 
 
+def make_pkg_requirement_from_addon_name(odoo_version_info, addon_name):
+    pkg_name = make_pkg_name(odoo_version_info, addon_name)
+    pkg_version_specifier = odoo_version_info.get("pkg_version_specifier", "")
+    return pkg_name + pkg_version_specifier
+
+
 def make_pkg_requirement(addon_dir, odoo_version_override=None):
     manifest = read_manifest(addon_dir)
     addon_name = os.path.basename(addon_dir)
     _, _, odoo_version_info = _get_version(
         addon_dir, manifest, odoo_version_override, git_post_version=False
     )
-    return make_pkg_name(odoo_version_info, addon_name)
+    return make_pkg_requirement_from_addon_name(odoo_version_info, addon_name)
 
 
 def _get_install_requires(
@@ -226,7 +263,9 @@ def _get_install_requires(
         if depends_override and depend in depends_override:
             install_require = depends_override[depend]
         else:
-            install_require = make_pkg_name(odoo_version_info, depend)
+            install_require = make_pkg_requirement_from_addon_name(
+                odoo_version_info, depend
+            )
         if install_require:
             install_requires.append(install_require)
     # python external_dependencies
@@ -238,7 +277,10 @@ def _get_install_requires(
             dep = external_dependencies_override.get("python", {})[dep]
         else:
             dep = external_dependencies.EXTERNAL_DEPENDENCIES_MAP.get(dep, dep)
-        install_requires.append(dep)
+        if isinstance(dep, list):
+            install_requires.extend(dep)
+        else:
+            install_requires.append(dep)
     return sorted(install_requires)
 
 
@@ -249,7 +291,7 @@ def get_install_requires_odoo_addon(
     external_dependencies_override=None,
     odoo_version_override=None,
 ):
-    """ Get the list of requirements for an addon """
+    """Get the list of requirements for an addon"""
     manifest = read_manifest(addon_dir)
     _, _, odoo_version_info = _get_version(
         addon_dir, manifest, odoo_version_override, git_post_version=False
@@ -269,7 +311,7 @@ def get_install_requires_odoo_addons(
     external_dependencies_override=None,
     odoo_version_override=None,
 ):
-    """ Get the list of requirements for a directory containing addons """
+    """Get the list of requirements for a directory containing addons"""
     addon_dirs = []
     addons = os.listdir(addons_dir)
     for addon in addons:
@@ -310,8 +352,12 @@ def _find_addons_dir():
     return res.pop()
 
 
-def _make_classifiers(manifest):
-    classifiers = ["Programming Language :: Python", "Framework :: Odoo"]
+def _make_classifiers(odoo_version, manifest):
+    classifiers = [
+        "Programming Language :: Python",
+        "Framework :: Odoo",
+        "Framework :: Odoo :: {}".format(odoo_version),
+    ]
 
     # commonly used licenses in OCA
     LICENSES = {
@@ -368,22 +414,36 @@ def _setuptools_find_packages(odoo_version_info):
     return pkg
 
 
+def _addon_name_from_metadata_name(metadata_name):
+    mo = METADATA_NAME_RE.match(metadata_name)
+    if not mo:
+        raise DistutilsSetupError(
+            "%s does not look like an Odoo addon package name" % metadata_name
+        )
+    return mo.group("addon_name")
+
+
 def get_addon_metadata(
     addon_dir,  # type: str
     depends_override=None,  # type: dict[str, str]
     external_dependencies_override=None,  # type: dict[str: dict[str: str]]
     odoo_version_override=None,  # type: str
     post_version_strategy_override=None,  # type: str
+    precomputed_metadata_path=None,  # type: str
 ):
     # type: (...) -> Message
     """
-    Return Python Package Metadata 2.1 for an Odoo addon as an
+    Return Python Package Metadata 2.2 for an Odoo addon as an
     email.message.Message.
 
     The Description field is absent and is stored in the message payload.
     All values are guaranteed to not contain newline characters, except for
     the payload. On python 3, all values are str. On python 2 the value types
     can be str or unicode depending on the python manifests.
+
+    ``precomputed_metadata_path`` may point to a file containing pre-computed
+    metadata that will be used to obtain the Name and Version, instead of looking
+    at the addon directory name or manifest version + VCS, respectively.
     """
     smeta = get_addon_setuptools_keywords(
         addon_dir,
@@ -391,6 +451,7 @@ def get_addon_metadata(
         external_dependencies_override=external_dependencies_override,
         odoo_version_override=odoo_version_override,
         post_version_strategy_override=post_version_strategy_override,
+        precomputed_metadata_path=precomputed_metadata_path,
     )
     meta = Message()
 
@@ -402,7 +463,7 @@ def get_addon_metadata(
             for item in svalue:
                 meta[name] = item
 
-    meta["Metadata-Version"] = "2.1"
+    meta["Metadata-Version"] = "2.2"
     _set("Name", "name")
     _set("Version", "version")
     _set("Requires-Python", "python_requires")
@@ -426,18 +487,20 @@ def get_addon_setuptools_keywords(
     external_dependencies_override=None,
     odoo_version_override=None,
     post_version_strategy_override=None,
+    precomputed_metadata_path=None,
 ):
-    addon_name = os.path.basename(os.path.abspath(addon_dir))
     manifest = read_manifest(addon_dir)
-    if os.path.exists("PKG-INFO"):
-        with io.open("PKG-INFO", encoding="utf-8") as fp:
+    if precomputed_metadata_path and os.path.exists(precomputed_metadata_path):
+        with io.open(precomputed_metadata_path, encoding="utf-8") as fp:
             pkg_info = email.parser.HeaderParser().parse(fp)
+            addon_name = _addon_name_from_metadata_name(pkg_info["Name"])
             version = pkg_info["Version"]
-        _, _, odoo_version_info = _get_version(
+        _, odoo_version, odoo_version_info = _get_version(
             addon_dir, manifest, odoo_version_override, git_post_version=False
         )
     else:
-        version, _, odoo_version_info = _get_version(
+        addon_name = os.path.basename(os.path.abspath(addon_dir))
+        version, odoo_version, odoo_version_info = _get_version(
             addon_dir,
             manifest,
             odoo_version_override,
@@ -465,7 +528,7 @@ def get_addon_setuptools_keywords(
         "python_requires": odoo_version_info["python_requires"],
         "author": _get_author(manifest),
         "author_email": _get_author_email(manifest),
-        "classifiers": _make_classifiers(manifest),
+        "classifiers": _make_classifiers(odoo_version, manifest),
     }
     # import pprint; pprint.pprint(setup_keywords)
     return {k: v for k, v in setup_keywords.items() if v is not None}
@@ -475,6 +538,7 @@ def prepare_odoo_addon(
     depends_override=None,
     external_dependencies_override=None,
     odoo_version_override=None,
+    post_version_strategy_override=None,
 ):
     addons_dir, addons_ns = _find_addons_dir()
     potential_addons = os.listdir(addons_dir)
@@ -507,9 +571,11 @@ def prepare_odoo_addon(
     addon_dir = os.path.join(addons_dir, addon_name)
     return get_addon_setuptools_keywords(
         addon_dir,
-        depends_override,
-        external_dependencies_override,
-        odoo_version_override,
+        depends_override=depends_override,
+        external_dependencies_override=external_dependencies_override,
+        odoo_version_override=odoo_version_override,
+        post_version_strategy_override=post_version_strategy_override,
+        precomputed_metadata_path="./PKG-INFO",
     )
 
 
